@echo off

if "%1"=="/?" (
    echo HostInfoTools-windows.cmd [/S] [/F] [/D]
    echo.
    echo   /S        Suppresses prompting to confirm you want to collect information.
    echo   /F        Collects full information including UltraPath logs and system eventlogs.
    echo   /D        Collects DUMP file.
    goto :eof
)

set WIN2003=52
set WIN2008R2=61

set CURDIR="%~dp0"
pushd %CURDIR%

setlocal ENABLEDELAYEDEXPANSION

call :get_system_version SYSVER

if %SYSVER% LSS %WIN2003% (
    echo The system version is too low. This tool does not support.
    goto quit
)

call :get_free_space FREESPACE
call :get_dump_space DUMPSPACE

set /a INFORSPACE = 30 + DUMPSPACE

if %FREESPACE% LSS %INFORSPACE% (
    echo The free space of disk %~d0 is less than the collect information. Can't collect information.
    pause
    goto quit
)

:analyze_parameter
if "%1"=="" goto :whether_dump_collect
if %1==/s set SILENTMODE=1
if %1==/S set SILENTMODE=1
if %1==/f set FULLMODE=1
if %1==/F set FULLMODE=1
if %1==/d set DUMPMODE=1
if %1==/D set DUMPMODE=1
shift
goto :analyze_parameter

:whether_dump_collect
if not defined DUMPMODE (
    echo Do you want to collect DUMP file?
    set /p ISCOLLECTDUMP="(y/n):"
    if !ISCOLLECTDUMP!==y set DUMPMODE=1
    if !ISCOLLECTDUMP!==Y set DUMPMODE=1
    if !ISCOLLECTDUMP!==n goto :prompt_note
    if !ISCOLLECTDUMP!==N goto :prompt_note
    goto :whether_dump_collect
)

:prompt_note
echo NOTE: We will collect some information as follows:
echo     1) System information
echo     2) Installed softwares
echo     3) HBA ^& iSCSI information
echo     4) Cluster information
echo     5) Disks/Luns managed by UltraPath
echo     6) UltraPath information
echo     7^) UltraPath logs and system eventlogs
echo     8^) Some important information
if defined DUMPMODE (
echo     9^) DUMP File
)
echo.

:whether_collect
if not defined SILENTMODE (
    echo Do you want to collect these information?
    set /p ISCOLLECT="(y/n):"
    if !ISCOLLECT!==y goto :start_collect
    if !ISCOLLECT!==Y goto :start_collect
    if !ISCOLLECT!==n goto :eof
    if !ISCOLLECT!==N goto :eof
    goto :whether_collect
)

:start_collect
md collectinfo

call :collect_userinfo
call :collect_systeminfo
call :collect_msinfo32
call :collect_mpioinfo
call :collect_softwareinfo
call :collect_iscsiinfo
call :collect_clusterinfo
call :collect_hbainfo
call :collect_ultrapathinfo
call :collect_systemeventlog
call :get_powershelleventlog
call :collect_driverinfo
call :collect_tasklistinfo
call :collect_setupapilog
call :collect_environmentvariable
call :collect_timeinfo
call :collect_serviceinfo
call :collect_wmicinfo
call :collect_storportinfo
call :collect_msiscsiinfo
call :collect_mpiompclaiminfo
call :seceditinfo

if defined DUMPMODE (
    CALL :collect_dump_file
)

call :get_system_time SYSTIME
7za.exe a "Windows-HostInfoOutput-%SYSTIME%.zip" ".\collectinfo\*"
rd /s /q collectinfo

:quit
popd
exit 0

:get_system_version
for /f "delims=" %%i in ('ver') do set ver=%%i
set ver=%ver:~-9,3%
set %1=%ver:.=%
goto :eof

:get_free_space
for /f "tokens=3" %%i in ('dir /-c %~d0') do set freespace=%%i
set %1=%freespace:~,-6%
goto :eof

:get_dump_space
for /f "tokens=3" %%i in ('dir /-c %SystemRoot%\MEMORY.DMP  ^| find /i "MEMORY.DMP"') do set dumpspace=%%i
set %1=%dumpspace:~,-6%
goto :eof

:get_system_time
for /f "tokens=1-2" %%i in ('echo %DATE%') do (
    echo %%i | findstr "[0-9]*/[0-9]*/[0-9]*" >nul 2>&1
    if !ERRORLEVEL! equ 0 (
        set curdate=%%i
        set curdate=!curdate:/=-!
    )
	echo %%i | findstr "[0-9]*-[0-9]*-[0-9]*" >nul 2>&1
    if !ERRORLEVEL! equ 0 (
        set curdate=%%i
    )
    echo %%j | findstr "[0-9]*/[0-9]*/[0-9]*" >nul 2>&1
    if !ERRORLEVEL! equ 0 (
        set curdate=%%j
        set curdate=!curdate:/=-!
    )
	echo %%j | findstr "[0-9]*-[0-9]*-[0-9]*" >nul 2>&1
    if !ERRORLEVEL! equ 0 (
        set curdate=%%j
    )
)

for /f "tokens=1-3 delims=:." %%i in ('echo %TIME%') do set curtime=%%i-%%j-%%k
set %1=%curdate%-%curtime%
goto :eof

:collect_userinfo
echo [username]%USERNAME%[/username] > collectinfo\userinfo.txt 2>&1
echo [syspath]%PATH%[/syspath] >> collectinfo\userinfo.txt 2>&1
net user >> collectinfo\userinfo.txt 2>&1
cacls . >> collectinfo\userinfo.txt 2>&1
goto :eof

:collect_systeminfo
systeminfo > collectinfo\systeminfo.txt 2>&1
goto :eof

:collect_msinfo32
if %SYSVER% LEQ %WIN2003% (
    "%COMMONPROGRAMFILES%\Microsoft Shared\MSInfo\msinfo32.exe" /categories +all /report collectinfo\winmsd.txt
) else (
    msinfo32.exe /categories +all /report collectinfo\winmsd.txt
)
goto :eof

:collect_mpioinfo
reg query HKLM\SYSTEM\CurrentControlSet\Control\Mpdev /v MPIOSupportedDeviceList /se # > collectinfo\mpio_mpdevsupporteddevicelist.txt 2>&1
reg query HKLM\SYSTEM\CurrentControlSet\services\msdsm\Parameters /v DsmSupportedDeviceList /se # > collectinfo\mpio_msdsmsupporteddevicelist.txt 2>&1
goto :eof

:collect_softwareinfo
reg query HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall /s > collectinfo\software.txt 2>&1
reg query HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall /s >> collectinfo\software.txt 2>&1
goto :eof

:collect_iscsiinfo
pushd collectinfo
md iscsi
iscsicli ListTargets > iscsi\iscsi_listtargets.txt 2>&1
iscsicli ListTargetPortals > iscsi\iscsi_listtargetportals.txt 2>&1
iscsicli ListPersistentTargets > iscsi\iscsi_listpersistenttargets.txt 2>&1
iscsicli ReportPersistentDevices > iscsi\iscsi_reportpersistentdevices.txt 2>&1
iscsicli ReportTargetMappings > iscsi\iscsi_reporttargetmappings.txt 2>&1
iscsicli ListInitiators > iscsi\iscsi_listinitiators.txt 2>&1
iscsicli ListiSNSServers > iscsi\iscsi_listisnsservers.txt 2>&1
iscsicli SessionList > iscsi\iscsi_sessionlist.txt 2>&1
iscsicli /help | findstr Microsoft  > iscsi\iscsi_version.txt 2>&1

set REGQUERYCMD=reg query HKLM\SYSTEM\CurrentControlSet\Control\Class\{4D36E97B-E325-11CE-BFC1-08002BE10318} /v /s /f Parameters /k
for /f "delims= " %%i in ('%REGQUERYCMD%') do (
    if /i not %%i==end reg query %%i /s >> iscsi\iscsi_parameters.txt 2>&1
)
popd
goto :eof

:collect_clusterinfo
pushd collectinfo
md cluster
reg query HKLM\Cluster /s > cluster\cluster_registry.txt 2>&1
if %SYSVER% LEQ %WIN2008R2% (
    cluster.exe res > cluster\cluster_resource.txt 2>&1
    cluster.exe res /priv > cluster\cluster_property.txt 2>&1
    copy /y %SystemRoot%\Cluster\*.log cluster\
    copy /y %SystemRoot%\Cluster\*.mht cluster\
    cluster.exe log /g
    xcopy /e /y %SystemRoot%\Cluster\Reports cluster\
) else (
    mode con:cols=200 lines=300
    powershell -command "& {Get-ClusterResource}" > cluster\cluster_resource.txt 2>&1
    powershell -command "& {Get-ClusterResource | Get-ClusterParameter}" > cluster\cluster_property.txt 2>&1
    powershell -command "get-clusterlog -uselocaltime"
    xcopy /e /y %SystemRoot%\Cluster\Reports cluster\
)
popd
goto :eof

:collect_hbainfo
cscript hba.vbs > collectinfo\hbainfo.txt 2>&1
goto :eof

:collect_ultrapathinfo
pushd collectinfo
md ultrapath
upadm show upconfig > ultrapath\ultrapath_config.txt 2>&1
upadm show version > ultrapath\ultrapath_version.txt 2>&1
upadm show array > ultrapath\ultrapath_array.txt 2>&1
upadm show vlun > ultrapath\ultrapath_vlun.txt 2>&1
upadm show path > ultrapath\ultrapath_path.txt 2>&1
upadm show alarmenable > ultrapath\ultrapath_alarmenable.txt 2>&1
upadm show path_reliability_enable > ultrapath\ultrapath_path_reliability_enable.txt 2>&1
upadm show vlun type=all > ultrapath\ultrapath_vlun_type_all.txt 2>&1
upadm check status > ultrapath\ultrapath_checkstatus.txt 2>&1
for /f "tokens=1" %%i in ('upadm show vlun ^| findstr /i "Disk"') do upadm show vlun id=%%i >> ultrapath\ultrapath_vlun_details.txt 2>&1

reg query HKLM\SYSTEM\CurrentControlSet\services\nxup\Parameters /v SetUpPhaseMark > ultrapath\ultrapath_install.txt 2>&1
reg query HKLM\SYSTEM\CurrentControlSet\services\nxup /s > ultrapath\nxup_reg.txt 2>&1

pushd ultrapath
   if /i %PROCESSOR_ARCHITECTURE%==x86 (
	call :collect_ultrapathlog "%PROGRAMFILES%\Huawei\OceanStor UltraPath" "ProgramFilesHuawei"
	call :collect_ultrapathlog "%PROGRAMFILES%\UltraPath" "ProgramFilesUltraPath"
   ) else (
	call :collect_ultrapathlog "%PROGRAMFILES(x86)%\Huawei\OceanStor UltraPath" "ProgramFilesHuawei"
	call :collect_ultrapathlog "%PROGRAMFILES(x86)%\UltraPath" "ProgramFilesUltraPath"
   )

call :collect_ultrapathlog "%SYSTEMDRIVE%\UltraPath" "UltraPath"
popd

popd
goto :eof

:collect_ultrapathlog
if exist %1 (
	xcopy /y /e %1\log log\%2\
	xcopy /y %1\bin\*.inf cfg\%2\
	xcopy /y %1\bin\*.conf cfg\%2\
	xcopy /y %1\bin\*.xml cfg\%2\
)
goto :eof

:collect_systemeventlog
pushd collectinfo
md systemeventlog
for %%i in (Application,HardwareEvents,System) do (
    call :get_eventlog_file %%i
)
call :get_powershelllog_file
popd
goto :eof

:get_eventlog_file
for /f "tokens=3" %%i in ('reg query HKLM\SYSTEM\CurrentControlSet\services\eventlog\%1 /v File') do (
    set EVENTLOGFILE="%%i"
    set EVENTLOGFILE=!EVENTLOGFILE:%%SystemRoot%%=%SystemRoot%!
)

pushd systemeventlog
copy /y %EVENTLOGFILE% .\
popd

goto :eof

:get_powershelleventlog
copy /y "%SystemRoot%\System32\winevt\Logs\Windows PowerShell.evtx" collectinfo\systemeventlog\
goto :eof

:collect_driverinfo
driverquery > collectinfo\driverinfo.txt 2>&1
goto :eof

:collect_tasklistinfo
tasklist > collectinfo\tasklistinfo.txt 2>&1
goto :eof

:collect_dump_file
xcopy /y %SystemRoot%\MEMORY.DMP collectinfo\dump\
goto :eof

:collect_setupapilog
xcopy /y %SystemRoot%\inf\setupapi.*.log collectinfo\
goto :eof

:collect_environmentvariable
set  > collectinfo\environmentvariableinfo.txt 2>&1
goto :eof

:collect_timeinfo
date /t  >> collectinfo\timeinfo.txt 2>&1
time /t  >> collectinfo\timeinfo.txt 2>&1
goto :eof

:collect_serviceinfo
net start  >> collectinfo\serviceinfo.txt 2>&1
sc query state= all  >> collectinfo\serviceinfo.txt 2>&1
goto :eof

:collect_wmicinfo
pushd collectinfo
md wmic
wmic cpu  > wmic\cpuinfo.txt 2>&1
wmic logicaldisk  > wmic\logicaldiskinfo.txt 2>&1
wmic qfe list /format:htable   > wmic\hotfixinfo.html
wmic memorychip  > wmic\memorychipinfo.txt 2>&1
popd
goto :eof

:collect_storportinfo
copy /y %SystemRoot%\System32\drivers\storport.sys collectinfo\storport.info
goto :eof

:collect_mpiompclaiminfo
mpclaim -v collectinfo\mpiompclaiminfo.txt
goto :eof

:collect_msiscsiinfo
copy /y %SystemRoot%\System32\drivers\msiscsi.sys collectinfo\msiscsi.info
goto :eof

:seceditinfo
secedit /export /cfg collectinfo\seceditinfo.txt
goto :eof